<?php

Class WebSocketServer {

    protected $address = null;
    protected $port = null;
    protected $users = array();
    protected $master = null;
    protected $sockets = array();
    protected $callback = null;

    public function __construct($address, $port, $callback) {
        $this->address = $address;
        $this->port = $port;
        $this->callback = $callback;
        $this->connectMaster($address, $port);
    }

    public function getUsers() {
        return $this->users;
    }

    protected function connectMaster() {
        // create a server socket.  AF_INET = IPv4, SOCK_STREAM + SOL_TCP = TCP/IP
        $this->master = socket_create(AF_INET, SOCK_STREAM, SOL_TCP) or die("socket_create() failed");
        // reuse local addresses
        socket_set_option($this->master, SOL_SOCKET, SO_REUSEADDR, 1) or die("socket_option() failed");
        // bind the IP address to the socket
        socket_bind($this->master, $this->address, $this->port) or die("socket_bind() failed");
        // listen for incoming connections (max of 20 pending)
        socket_listen($this->master, 20) or die("socket_listen() failed");
        // save the socket object
        $this->sockets[] = $this->master;

        $this->log("Websocket server started : ". date('Y-m-d H:i:s'));
        $this->log("Listening on             : ". $this->address ." port ". $this->port);
        $this->log("Master socket            : ". $this->master);
        $this->log("CTRL-C to quit");

        return $this->master;
    }

    public function run() {
        // main run loop
        while(true){
            $changed = $this->sockets;
            // wait for any of the sockets to change (have data to read)
            // see http://www.php.net/manual/en/function.socket-select.php
            if (false === socket_select($changed, $write=NULL, $except=NULL,NULL)) {
                die("socket_select() failed: ". socket_strerror(socket_last_error()));
            }
            foreach($changed as $socket){
                if ($socket == $this->master) {
                    // new incoming connection, time to accept
                    $this->log("Incoming connection request");
                    $client = socket_accept($this->master);
                    if($client === false){ $this->log("socket_accept() failed: ". socket_strerror(socket_last_error())); continue; }
                    else{ $this->connect($client); }
                } else {
                    // existing connection has data to read
                    $bytes = @socket_recv($socket, $buffer, 2048, 0);
                    // close the connection if there was an error or no data to read
                    if($bytes == 0) {
                        $this->disconnect($socket);
                    } else {
                        $user = $this->getUserBySocket($socket);
                        // perform the WebSocket upgrade HTTP handshake if needed
                        if(! $user->handshake) {
                            $this->doHandshake($buffer, $socket);
                            $user->handshake = true;
                        } else {
                            $this->log("< ". $this->unwrap($buffer));
                            $user->lastAction = time();
                            // call the callback function
                            if ($this->callback) {
                                call_user_func($this->callback, $user, $this->unwrap($buffer), $this);
                            }
                        }
                    }
                }
            }
        }
    }

    private function doHandshake($buffer, &$socket) {
        // parse the HTTP headers
        list($resource, $headers, $securityCode) = $this->handleRequestHeader($buffer);

        // you could add logic here based on the resource requested
        $this->log("client requested resource: $resource");

        // answer the security challenge correctly
        $securityResponse = '';
        if (isset($headers['Sec-WebSocket-Key1']) && isset($headers['Sec-WebSocket-Key2'])) {
            $securityResponse = $this->getHandshakeSecurityKey($headers['Sec-WebSocket-Key1'], $headers['Sec-WebSocket-Key2'], $securityCode);
        }

        if ($securityResponse) {
            // WebSocket draft version 76
            // http://tools.ietf.org/html/draft-hixie-thewebsocketprotocol-76
            $upgrade  = "HTTP/1.1 101 Web Socket Protocol Handshake\r\n" .
                "Upgrade: WebSocket\r\n" .
                "Connection: Upgrade\r\n" .
                "Sec-WebSocket-Origin: " . $headers['Origin'] . "\r\n" .
                "Sec-WebSocket-Location: ws://" . $headers['Host'] . $resource . "\r\n" .
                "\r\n".$securityResponse;
        } else {
            // WebSocket draft version 75
            // http://tools.ietf.org/html/draft-hixie-thewebsocketprotocol-75
            $upgrade  = "HTTP/1.1 101 Web Socket Protocol Handshake\r\n" .
                "Upgrade: WebSocket\r\n" .
                "Connection: Upgrade\r\n" .
                "WebSocket-Origin: " . $headers['Origin'] . "\r\n" .
                "WebSocket-Location: ws://" . $headers['Host'] . $resource . "\r\n" .
                "\r\n";
        }

        // send it out
        socket_write($socket, $upgrade, strlen($upgrade));

        return true;
    }

    private function handleSecurityKey($key) {
        // extract the numeric digits to form a number
        preg_match_all('/[0-9]/', $key, $number);
        // extract the number of spaces
        preg_match_all('/ /', $key, $space);
        if ($number && $space) {
            // perform the calculation per the websocket spec
            return implode('', $number[0]) / count($space[0]);
        }
        return '';
    }

    private function getHandshakeSecurityKey($key1, $key2, $code) {
        return md5(
            pack('N', $this->handleSecurityKey($key1)).
            pack('N', $this->handleSecurityKey($key2)).
            $code,
            true
        );
    }

    private function handleRequestHeader($request) {
        $resource = $code = null;
        // get the resource requested (part of URL after domain and port)
        preg_match('/GET (.*?) HTTP/', $request, $match) && $resource = $match[1];
        // get the security code if present (WebSockets draft 76)
        preg_match("/\r\n(.*?)\$/", $request, $match) && $code = $match[1];

        // remember the rest of the headers
        $headers = array();
        foreach(explode("\r\n", $request) as $line) {
            if (strpos($line, ': ') !== false) {
                list($key, $value) = explode(': ', $line);
                $headers[trim($key)] = trim($value);
            }
        }
        return array($resource, $headers, $code);
    }

    public function connect(&$socket) {
        $this->log($socket." CONNECTED");
        $this->users[] = new WebSocketUser($socket);
        $this->sockets[] = $socket;
    }

    public function disconnect(&$socket) {
        if ($this->users) {
            $found=null;
            $n = count($this->users);
            for($i=0;$i<$n;$i++){
                if($this->users[$i]->socket == $socket){ $found=$i; break; }
            }
            if(!is_null($found)){ array_splice($this->users, $found, 1); }
            $index = array_search($socket, $this->sockets);
            socket_close($socket);
            $this->log($socket." DISCONNECTED!");
            if($index>=0){ array_splice($this->sockets, $index, 1); }
        }
    }

    public function getUserBySocket($socket) {
        $found=null;
        foreach($this->users as $user) {
            if($user->socket==$socket) {
                $found=$user;
                break;
            }
        }
        return $found;
    }

    public function send($client, $msg){
        $this->log("> ".$msg);
        $msg = $this->wrap($msg);
        @socket_write($client, $msg, strlen($msg));
    }

    private function log($msg="") { echo $msg."\n"; }
    private function wrap($msg="") { return chr(0).$msg.chr(255); }
    private function unwrap($msg="") { return substr($msg, 1, strlen($msg)-2); }
}


/**
 * WebSocketUser Class
 */
class WebSocketUser {

    public $id = null;
    public $socket = null;
    public $handshake = false;
    public $ip = null;
    public $lastAction = null;
    public $data = array();

    public function __construct(&$socket) {
        $this->id = uniqid();
        $this->socket = $socket;

        socket_getpeername($socket, $ip);
        $this->ip = $ip;
    }
}

